from izzylib import (
	BaseConfig,
	LowerDotDict
)

from .request import ClientRequest
from .response import ClientResponse

from .. import __version__
from ..utils import AsyncTransport


class Config(BaseConfig):
	def __init__(self, **kwargs):
		super().__init__(
			headers = LowerDotDict({'User-Agent': f'IzzyLib-HTTP-Async/{__version__}'}),
			request_class = ClientRequest,
			response_class = ClientResponse,
			transport_class = AsyncTransport,
			timeout = 60
		)

		useragent = kwargs.pop('useragent', None)
		appagent = kwargs.pop('appagent', None)

		if useragent:
			self.headers['User-Agent'] = useragent
			appagent = None

		if appagent:
			self.headers['User-Agent'] = f'IzzyLib/{__version__} ({appagent})'

		self.update(kwargs)


	@property
	def agent(self):
		return self.headers['User-Agent']
