# todo: rename any *Exception class to *Error

__all__ = [
	'HttpError',
	'HttpFileDownloadedError',
	'InvalidMethodException',
	'MethodNotHandledException',
	'NoBlueprintForPath',
	'NoConnectionError',
	'MaxConnectionsError'
]


class HttpError(Exception):
	def __init__(self, status, message):
		self.status = status
		self.message = message

		super().__init__(f'HTTP ERROR {status}: {message[:100]}')


class HttpFileDownloadedError(Exception):
	'raise when a download failed for any reason'


class InvalidMethodException(Exception):
	def __init__(self, method):
		super().__init__(f'Invalid HTTP method: {method}')
		self.method = method


class MethodNotHandledException(Exception):
	def __init__(self, method):
		super().__init__(f'HTTP method not handled by handler: {method}')
		self.method = method


class NoBlueprintForPath(Exception):
	'raise when no blueprint is found for a specific path'


class NoConnectionError(Exception):
	'Raise when a function requiring a connection gets called when there is no connection'


class MaxConnectionsError(Exception):
	'Raise when the max amount of connections has been reached'
