from functools import partial


class HttpError(Exception):
	def __init__(self, message, status=500):
		super().__init__(f'HTTP Error {status}: {message}')

		self.status = status
		self.message = message


class RedirError(Exception):
	def __init__(self, path, status=301):
		super().__init__(f'HTTP Error {status}: {path}')

		self.status = status
		self.path = path


## 200 Errors
Ok = partial(HttpError, status=200)
Created = partial(HttpError,status=201)
Accepted = partial(HttpError,status=202)
NoContent = partial(HttpError,status=204)
ResetContent = partial(HttpError,status=205)
PartialContent = partial(HttpError,status=206)

## 300 Errors
NotModified = partial(HttpError, status=304)

## 400 Errors
BadRequest = partial(HttpError, status=400)
Unauthorized = partial(HttpError, status=401)
Forbidden = partial(HttpError, status=403)
NotFound = partial(HttpError, status=404)
MethodNotAllowed = partial(HttpError, status=405)
RequestTimeout = partial(HttpError,status=408)
Gone = partial(HttpError,status=410)
LengthRequired = partial(HttpError,status=411)
PreconditionFailed = partial(HttpError,status=412)
PayloadTooLarge = partial(HttpError,status=413)
UriTooLong = partial(HttpError,status=414)
UnsupportedMediaType = partial(HttpError,status=415)
RangeNotSatisfiable = partial(HttpError,status=416)
Teapot = partial(HttpError, status=418)
UpgradeRequired = partial(HttpError,status=426)
TooManyRequests = partial(HttpError,status=429)
RequestHeaderFieldsTooLarge = partial(HttpError,status=431)
UnavailableForLegalReasons = partial(HttpError,status=451)

## 500 Errors
InternalServerError = partial(HttpError, status=500)
NotImplemented = partial(HttpError,status=501)
BadGateway = partial(HttpError,status=502)
ServiceUnavailable = partial(HttpError,status=503)
GatewayTimeout = partial(HttpError,status=504)
HttpVersionNotSuported = partial(HttpError,status=505)
NetworkAuthenticationRequired = partial(HttpError,status=511)

## Redirects
MovedPermanently = partial(RedirError, status=301)
Found = partial(RedirError, status=302)
SeeOther = partial(RedirError, status=303)
TemporaryRedirect = partial(RedirError, status=307)
PermanentRedirect = partial(RedirError, status=309)
