import mimetypes

from izzylib import Color, DotDict, Path

from . import http_methods, error

from ..exceptions import (
	InvalidMethodException,
	MethodNotHandledException
)

try: import magic
except ImportError: magic = None


default_theme = DotDict(
	primary = Color('#e7a'),
	secondary = Color('#a7e'),
	background = Color('#191919'),
	positive = Color('#aea'),
	negative = Color('#e99'),
	white = Color('#eee'),
	black = Color('#111'),
	speed = 250
)


class View:
	__path__ = ''

	def __init__(self, app):
		self.app = app


	def get_handler(self, method):
		if method.upper() not in http_methods:
			raise InvalidMethodException(method)

		try:
			return getattr(self, method.lower())
		except AttributeError:
			raise MethodNotHandledException(method)


	#def get(self, request):
		#pass


def Static(src):
	src = Path(src)

	async def StaticHandler(request, response, path=None):
		src_path = src if not path else src.join(path)

		try:
			with open(src_path, 'rb') as fd:
				data = fd.read()

				if magic:
					mime = mimetypes.guess_type(path)[0] or magic.from_buffer(data[:2048], mime=True)

				else:
					mime = mimetypes.guess_type(path)

		except FileNotFoundError:
			raise error.NotFound('Static file not found')

		except IsADirectoryError:
			index = src_path.join('index.html')

			if not index.isfile:
				raise error.NotFound('Static file not found')

			with open(index, 'rb') as fd:
				data = fd.read()
				mime = 'text/html'

		response.body = data
		response.content_type = mime

	return StaticHandler


### Frontend Template Views ###

class Manifest(View):
	__path__ = '/framework/manifest.json'


	async def get(self, request, response):
		data = {
			'name': self.app.cfg.name,
			'short_name': self.app.cfg.name.replace(' ', ''),
			'description': 'UvU',
			'icons': [
				{
					'src': "/framework/static/icon512.png",
					'sizes': '512x512',
					'type': 'image/png'
				},
				{
					'src': "/framework/static/icon64.png",
					'sizes': '64x64',
					'type': 'image/png'
				}
			],
			'theme_color': str(default_theme.primary),
			'background_color': str(default_theme.background),
			'display': 'standalone',
			'start_url': '/',
			'scope': f'{self.app.cfg.proto}://{self.app.cfg.web_host}'
		}

		response.set_json(data)


class Robots(View):
	__path__ = '/robots.txt'

	async def get(self, request, response):
		data = '# Disallow all\nUser-agent: *\nDisallow: /'
		response.body = data


class Style(View):
	__path__ = '/framework/style.css'

	async def get(self, request, response):
		response.body = self.app.render('base.css', default_theme)
		response.content_type = 'text/css'
